"use strict";

exports.__esModule = true;
exports["default"] = buildChildren;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _cleanText = require("./clean-text");

var _cleanText2 = _interopRequireDefault(_cleanText);

var _astToFunctionCall = require("./ast/to-function-call");

var _astToFunctionCall2 = _interopRequireDefault(_astToFunctionCall);

var _runtimeRenderArbitrary = require("./runtime/render-arbitrary");

// Filters out empty children, and transform JSX expressions
// into function calls.

var _runtimeRenderArbitrary2 = _interopRequireDefault(_runtimeRenderArbitrary);

function buildChildren(t, scope, file, children, eager) {
  var renderArbitraryRef = undefined;

  return children.reduce(function (children, child) {
    var wasExpressionContainer = t.isJSXExpressionContainer(child);
    if (wasExpressionContainer) {
      child = child.expression;
    }

    if (t.isJSXEmptyExpression(child)) {
      return children;
    }

    if (t.isLiteral(child)) {
      var type = typeof child.value;
      var value = child.value;
      if (type === "string") {
        value = _cleanText2["default"](value);
        if (!value) {
          return children;
        }
      }

      if (type === "string" || type === "number") {
        child = _astToFunctionCall2["default"](t, "text", [t.literal(value)]);
      }
    } else if (wasExpressionContainer && t.isExpression(child) && !child._wasJSX) {
      renderArbitraryRef = renderArbitraryRef || _runtimeRenderArbitrary2["default"](t, file);
      if (eager) {
        var ref = scope.generateUidIdentifierBasedOnNode(child);
        children.push(t.variableDeclaration("let", [t.variableDeclarator(ref, child)]));
        child = ref;
      }
      child = _astToFunctionCall2["default"](t, renderArbitraryRef, [child]);
    }

    children.push(child);
    return children;
  }, []);
}

module.exports = exports["default"];