"use strict";

exports.__esModule = true;
exports["default"] = extractOpenArguments;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _astToReference = require("./ast/to-reference");

// Extracts attributes into the appropriate
// attribute array. Static attributes and the key
// are placed into static attributes, and expressions
// are placed into the variadic attributes.

var _astToReference2 = _interopRequireDefault(_astToReference);

function extractOpenArguments(t, scope, attributes, eager) {
  var key = null;
  var statics = [];
  var attrs = [];
  var attributeDeclarators = [];
  var hasSpread = false;

  for (var _iterator = attributes, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref = _i.value;
    }

    var attribute = _ref;

    if (t.isJSXSpreadAttribute(attribute)) {
      hasSpread = true;
      attrs.push(attribute);
      continue;
    }

    var attr = _astToReference2["default"](t, attribute.name);
    var _name = attr.value;
    var value = attribute.value;

    if (t.isJSXExpressionContainer(value)) {
      value = value.expression;
      if (eager && !t.isLiteral(value) && !value._wasJSX) {
        var ref = scope.generateUidIdentifierBasedOnNode(value);
        attributeDeclarators.push(t.variableDeclarator(ref, value));
        value = ref;
      }
    } else if (!value) {
      value = t.literal(true);
    }

    if (_name === "key") {
      key = value;
    }

    if (_name === "key" || t.isLiteral(value)) {
      statics.push(attr, value);
    } else {
      attrs.push([attr, value]);
    }
  }

  if (!statics.length) {
    statics = null;
  }
  if (!attrs.length) {
    attrs = null;
  }

  return { key: key, statics: statics, attrs: attrs, attributeDeclarators: attributeDeclarators, hasSpread: hasSpread };
}

module.exports = exports["default"];