"use strict";

exports.__esModule = true;
exports["default"] = injectRenderArbitrary;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _inject = require("../inject");

var _inject2 = _interopRequireDefault(_inject);

var _forOwn = require("./for-own");

var _forOwn2 = _interopRequireDefault(_forOwn);

var _astToFunctionCallStatement = require("../ast/to-function-call-statement");

// Isolated AST code to determine if a value is textual
// (strings and numbers).

var _astToFunctionCallStatement2 = _interopRequireDefault(_astToFunctionCallStatement);

function isTextual(t, type, value) {
  return t.binaryExpression("||", t.binaryExpression("===", type, t.literal("number")), t.binaryExpression("||", t.binaryExpression("===", type, t.literal("string")), t.binaryExpression("&&", value, t.binaryExpression("instanceof", value, t.identifier("String")))));
}

// Isolated AST code to determine if a value is a wrapped
// DOM manipulator function.
function isDOMWrapper(t, type, value) {
  return t.binaryExpression("&&", t.binaryExpression("===", type, t.literal("function")), t.memberExpression(value, t.identifier("__jsxDOMWrapper")));
}

// Isolated AST code to determine if a value an Array.
function isArray(t, value) {
  return t.callExpression(t.memberExpression(t.identifier("Array"), t.identifier("isArray")), [value]);
}

// Renders an arbitrary JSX Expression into the DOM.
// Valid types are strings, numbers, and DOM manipulators
// (which will be wrapped).
// It may also be an Array or Object, which will be iterated
// recursively.
// Depends on the _forOwn helper.
function renderArbitraryAST(t, ref, deps) {
  var forOwn = deps.forOwn;
  var child = t.identifier("child");
  var type = t.identifier("type");
  var forEach = t.memberExpression(child, t.identifier("forEach"));

  /**
   * function _renderArbitrary(child) {
   *   var type = typeof child;
   *   if (type === "number" || (type === string || child && child instanceof String)) {
   *     text(child);
   *   } else if (type === "function" && child.__jsxDOMWrapper) {
   *     child();
   *   } else if (Array.isArray(child)) {
   *     child.forEach(_renderArbitrary);
   *   } else {
   *     _forOwn(child, _renderArbitrary);
   *   }
   * }
   */
  return t.functionDeclaration(ref, [child], t.blockStatement([t.variableDeclaration("var", [t.variableDeclarator(type, t.unaryExpression("typeof", child))]), t.IfStatement(isTextual(t, type, child), t.blockStatement([_astToFunctionCallStatement2["default"](t, "text", [child])]), t.ifStatement(isDOMWrapper(t, type, child), t.blockStatement([_astToFunctionCallStatement2["default"](t, child, [])]), t.ifStatement(isArray(t, child), t.blockStatement([_astToFunctionCallStatement2["default"](t, forEach, [ref])]), t.blockStatement([_astToFunctionCallStatement2["default"](t, forOwn, [child, ref])]))))]));
}

function injectRenderArbitrary(t, file) {
  var forcedRef = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];

  return _inject2["default"](t, file, forcedRef, "renderArbitrary", renderArbitraryAST, {
    forOwn: _forOwn2["default"]
  });
}

module.exports = exports["default"];