"use strict";

exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _helpersAstToFunctionCall = require("./helpers/ast/to-function-call");

var _helpersAstToFunctionCall2 = _interopRequireDefault(_helpersAstToFunctionCall);

var _helpersAstToReference = require("./helpers/ast/to-reference");

var _helpersAstToReference2 = _interopRequireDefault(_helpersAstToReference);

var _helpersAttributesToAttrCalls = require("./helpers/attributes-to-attr-calls");

var _helpersAttributesToAttrCalls2 = _interopRequireDefault(_helpersAttributesToAttrCalls);

var _helpersBuildChildren = require("./helpers/build-children");

var _helpersBuildChildren2 = _interopRequireDefault(_helpersBuildChildren);

var _helpersExtractOpenArguments = require("./helpers/extract-open-arguments");

var _helpersExtractOpenArguments2 = _interopRequireDefault(_helpersExtractOpenArguments);

var _helpersFilterEagerDeclarators = require("./helpers/filter-eager-declarators");

var _helpersFilterEagerDeclarators2 = _interopRequireDefault(_helpersFilterEagerDeclarators);

var _helpersFindOtherJsx = require("./helpers/find-other-jsx");

var _helpersFindOtherJsx2 = _interopRequireDefault(_helpersFindOtherJsx);

var _helpersFlattenExpressions = require("./helpers/flatten-expressions");

var _helpersFlattenExpressions2 = _interopRequireDefault(_helpersFlattenExpressions);

var _helpersPartitionDeclarators = require("./helpers/partition-declarators");

var _helpersPartitionDeclarators2 = _interopRequireDefault(_helpersPartitionDeclarators);

var _helpersStatementsWithReturnLast = require("./helpers/statements-with-return-last");

var _helpersStatementsWithReturnLast2 = _interopRequireDefault(_helpersStatementsWithReturnLast);

var _helpersInject = require("./helpers/inject");

var _helpersRuntimeJsxWrapper = require("./helpers/runtime/jsx-wrapper");

var _helpersRuntimeJsxWrapper2 = _interopRequireDefault(_helpersRuntimeJsxWrapper);

exports["default"] = function (_ref2) {
  var Plugin = _ref2.Plugin;
  var t = _ref2.types;

  return new Plugin("incremental-dom", { visitor: {
      Program: _helpersInject.setupInjector,

      JSXElement: {
        enter: function enter(node, parent) {
          var inAssignment = false;
          var inAttribute = false;
          var inCallExpression = false;
          var inCollection = false;
          var inReturnStatement = false;
          var containingJSXElement = undefined;

          this.findParent(function (path) {
            if (path.isJSXElement()) {
              containingJSXElement = path;
              return true;
            }
            if (path.isFunction() || path.isProgram()) {
              return true;
            }
            if (path.isJSXAttribute()) {
              inAttribute = true;
            } else if (path.isAssignmentExpression() || path.isVariableDeclarator()) {
              inAssignment = true;
            } else if (path.isArrayExpression() || path.isObjectExpression()) {
              inCollection = true;
            } else if (path.isReturnStatement()) {
              inReturnStatement = true;
            } else if (path.isCallExpression()) {
              inCallExpression = true;
            }
          });

          // Values are useless if they aren't assigned.
          // ```
          //   var a = 1;
          //   <div /> // Useless JSX node
          // ```
          if (!(inReturnStatement || inAssignment || inCallExpression || containingJSXElement)) {
            this.dangerouslyRemove();
            return;
          }

          var needsWrapper = inAttribute || inAssignment || inCollection || inCallExpression;
          var containerNeedsWrapper = containingJSXElement ? containingJSXElement.getData("containerNeedsWrapper") || containingJSXElement.getData("needsWrapper") : false;

          if (!needsWrapper && !containingJSXElement) {
            // Determine if there are JSXElements in a higher scope.
            needsWrapper = _helpersFindOtherJsx2["default"](this);
          }

          // Tie a child JSXElement's eager declarations with the parent's, so
          // so all declarations come before the element.
          var eagerDeclarators = containingJSXElement ? containingJSXElement.getData("eagerDeclarators") : [];

          this.setData("containerNeedsWrapper", containerNeedsWrapper);
          this.setData("containingJSXElement", containingJSXElement);
          this.setData("eagerDeclarators", eagerDeclarators);
          this.setData("needsWrapper", needsWrapper);
        },

        exit: function exit(node, parent, scope, file) {
          var _data = this.data;
          var containerNeedsWrapper = _data.containerNeedsWrapper;
          var containingJSXElement = _data.containingJSXElement;
          var eagerDeclarators = _data.eagerDeclarators;
          var needsWrapper = _data.needsWrapper;

          var eager = needsWrapper || containerNeedsWrapper;

          // Filter out empty children, and transform JSX expressions
          // into normal expressions.
          var openingElement = node.openingElement;
          var closingElement = node.closingElement;
          var children = _helpersBuildChildren2["default"](t, scope, file, node.children, eager);

          var elements = [openingElement].concat(children);
          if (closingElement) {
            elements.push(closingElement);
          }

          // If we're inside a JSX node, flattening expressions
          // may force us into an unwanted function scope.
          if (t.isJSXElement(parent)) {
            // If the parent needs to be wrapped, we need to place our eager
            // child expressions outside the wrapper.
            if (containerNeedsWrapper) {
              elements = _helpersFilterEagerDeclarators2["default"](t, elements, eagerDeclarators);
            }
            return elements;
          }

          // Expressions Containers must contain an expression and not statements.
          // This will be flattened out into statements later.
          if (containingJSXElement && !needsWrapper) {
            var sequence = t.sequenceExpression(elements);
            // Mark this sequence as a JSX Element so we can avoid an unnecessary
            // renderArbitrary call.
            sequence._wasJSX = true;
            return sequence;
          }

          // Transform (recursively) any sequence expressions into a series of
          // statements.
          elements = _helpersFlattenExpressions2["default"](t, elements);

          // If we need to wrap this JSX element in a wrapper, we must place
          // any eagerly evaluated child expressions outside the wrapper.
          if (needsWrapper) {
            elements = _helpersFilterEagerDeclarators2["default"](t, elements, eagerDeclarators);
          }

          if (eagerDeclarators.length && !containingJSXElement) {
            // Find the closest statement, and insert our eager declarations
            // before it.
            var parentStatement = this.findParent(function (path) {
              return path.isStatement();
            });
            var declaration = t.variableDeclaration("let", eagerDeclarators);

            var _parentStatement$insertBefore = parentStatement.insertBefore(declaration);

            // Add our eager declarations to the scopes tracked bindings.
            var path = _parentStatement$insertBefore[0];
            scope.registerBinding("let", path);
          }

          // Ensure the last statement returns the DOM element.
          elements = _helpersStatementsWithReturnLast2["default"](t, elements);

          if (needsWrapper) {
            // Create a wrapper around our element, and mark it as a one so later
            // child expressions can identify and "render" it.
            var jsxWrapperRef = _helpersRuntimeJsxWrapper2["default"](t, file);
            var wrapper = _helpersAstToFunctionCall2["default"](t, jsxWrapperRef, [t.functionExpression(null, [], t.blockStatement(elements))]);
            wrapper._wasJSX = true;
            return wrapper;
          }

          // This is the main JSX element. Replace the return statement
          // will all the nested calls, returning the main JSX element.
          this.parentPath.replaceWithMultiple(elements);
        }
      },

      JSXOpeningElement: {
        exit: function exit(node, parent, scope, file) {
          var _JSXElement$getData;

          var tag = _helpersAstToReference2["default"](t, node.name);
          var args = [tag];
          var elementFunction = node.selfClosing ? "elementVoid" : "elementOpen";

          // Only eagerly evaluate our attributes if we will be wrapping the element.
          var JSXElement = this.parentPath;
          var eager = JSXElement.getData("needsWrapper") || JSXElement.getData("containerNeedsWrapper");

          var _extractOpenArguments = _helpersExtractOpenArguments2["default"](t, scope, node.attributes, eager);

          // Push any eager attribute declarators onto the element's list of
          // eager declarations.
          var key = _extractOpenArguments.key;
          var statics = _extractOpenArguments.statics;
          var attrs = _extractOpenArguments.attrs;
          var attributeDeclarators = _extractOpenArguments.attributeDeclarators;
          var hasSpread = _extractOpenArguments.hasSpread;
          (_JSXElement$getData = JSXElement.getData("eagerDeclarators")).push.apply(_JSXElement$getData, attributeDeclarators);

          // Only push arguments if they're needed
          if (key || statics) {
            args.push(key || t.literal(null));
          }
          if (statics) {
            args.push(t.arrayExpression(statics));
          }

          // If there is a spread element, we need to use
          // the elementOpenStart/elementOpenEnd syntax.
          // This allows spreads to be transformed into
          // attr(name, value) calls.
          if (hasSpread) {
            attrs = _helpersAttributesToAttrCalls2["default"](t, file, attrs);

            var expressions = [_helpersAstToFunctionCall2["default"](t, "elementOpenStart", args)].concat(attrs, [_helpersAstToFunctionCall2["default"](t, "elementOpenEnd", [tag])]);
            if (node.selfClosing) {
              expressions.push(_helpersAstToFunctionCall2["default"](t, "elementClose", [tag]));
            }

            return t.sequenceExpression(expressions);
          }

          if (attrs) {

            // Only push key and statics if they have not
            // already been pushed.
            if (!statics) {
              if (!key) {
                args.push(t.literal(null));
              }
              args.push(t.literal(null));
            }

            for (var _iterator = attrs, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
              var _ref;

              if (_isArray) {
                if (_i >= _iterator.length) break;
                _ref = _iterator[_i++];
              } else {
                _i = _iterator.next();
                if (_i.done) break;
                _ref = _i.value;
              }

              var _name = _ref[0];
              var value = _ref[1];

              args.push(_name, value);
            }
          }

          return _helpersAstToFunctionCall2["default"](t, elementFunction, args);
        }
      },

      JSXClosingElement: {
        exit: function exit(node) {
          return _helpersAstToFunctionCall2["default"](t, "elementClose", [_helpersAstToReference2["default"](t, node.name)]);
        }
      }
    } });
};

module.exports = exports["default"];